/*
 * @Copyright 2010, MechSoft 
 * MechSoft, Mechanical and Software Solutions 
 * 
 * Licensed under the Apache License, Version 2.0 (the 
 * "License"); you may not use this file except in compliance 
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.defne.jsf;

import javax.annotation.PreDestroy;

import org.defne.service.IMessageConstants;
import org.defne.service.Message;
import org.defne.service.MessageFactory;
import org.defne.service.executor.IServiceExecutor;
import org.defne.service.executor.IServiceExecutorProxy;
import org.defne.service.executor.ServiceExecutorProxyProvider;
import org.defne.service.executor.ServiceExecutorProxyTypes;
import org.defne.service.util.ServiceUtility;
import org.defne.utility.exception.DefneException;
import org.defne.utility.log.ILogger;
import org.defne.utility.log.LoggerProvider;

/**
 * JSF bean that is responsible for
 * executing the service requests.
 * 
 * <p>
 * This bean is a proxy and it propogates the service
 * execution requests to the actual EJB service executor
 * instance.
 * </p>
 * 
 * @author gurkanerdogdu
 *
 */
public class JSFEjbServiceExecutorProxy implements IServiceExecutorProxy
{
    /**Logger instance*/
    private static final ILogger logger = LoggerProvider.getLogProvider(JSFEjbServiceExecutorProxy.class);
    
    /**Service executor proxy jndi name*/
    private final String jndiName;
    
    /**Actual service executor proxy instance.*/
    private final IServiceExecutorProxy proxy;
    
    /**
     * Creates a new bean instance.
     */
    public JSFEjbServiceExecutorProxy()
    {
        this.jndiName = JSFUtility.getInitParameter(IServiceExecutor.SERVICE_EXECUTOR_JNDI_INIT_PARAMETER);
        this.proxy = ServiceExecutorProxyProvider.getServiceExecutorProxy(ServiceExecutorProxyTypes.EJB);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Message execute(Message inputBag)
    {
        Message outputBag = MessageFactory.newMessage(inputBag.getServiceName(), inputBag.getServiceMethodName());
        try
        {
            inputBag.putMessageParameter(IMessageConstants.SERVICE_EXECUTOR_EJB_JNDI_NAME_KEY, this.jndiName);
            outputBag = this.proxy.execute(inputBag);
        }
        catch (DefneException exception)
        {
            if(logger.isErrorEnabled())
            {
                logger.error(ServiceUtility.getErrorMessage(outputBag,exception),exception.getCause());                
                outputBag = ServiceUtility.getServiceExecutionError(outputBag, exception);
            }
        }
        
        return outputBag;
    }
    
    /**
     * Clear the bean context.
     */
    @PreDestroy
    public void destroy()
    {
        clear(this.jndiName);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void clear(Object clearObject)
    {
        this.proxy.clear(clearObject);
    }
}
